////////////////////////////////////////////////////////////////////////////////
//
// Description: Small cloth physics tutorial. Thats used to show basic physics
//              and instability due to the use of spring forces.
// Author     : Jens Schbel
// Web        : http://www.juliettesierra.de
// Company    : http://www.crytek.com
//
////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>

#include "../TinyXML/tinyxml.h"

#include "engine.h"

////////////////////////////////////////////////////////////////////////////////
PhyskoEngine::PhyskoEngine(const char* filename)
{
  LoadData(filename);
}

////////////////////////////////////////////////////////////////////////////////
PhyskoEngine::~PhyskoEngine()
{
  m_objectsList.clear();  
}

////////////////////////////////////////////////////////////////////////////////
void PhyskoEngine::ProcessPhysics(void)
{
  for(std::vector<IObject*>::iterator iterator = m_objectsList.begin();
    iterator != m_objectsList.end();
    ++iterator)
  {
    (*iterator)->ProcessPhysics();
  }
}

////////////////////////////////////////////////////////////////////////////////
void PhyskoEngine::ProcessRenderer(void)
{
  for(std::vector<IObject*>::iterator iterator = m_objectsList.begin();
    iterator != m_objectsList.end();
    ++iterator)
  {
    (*iterator)->ProcessRenderer();
  }
}

////////////////////////////////////////////////////////////////////////////////
void PhyskoEngine::Loop(void)
{
  for (int i = 0; i < 1; ++i)
  {
    ProcessPhysics();
    ProcessRenderer();
  }
}

////////////////////////////////////////////////////////////////////////////////
void PhyskoEngine::LoadData(const char* filename)
{
  TiXmlDocument parameterXMLDoc = TiXmlDocument(filename);
	if ( !parameterXMLDoc.LoadFile() )
	{
		printf( "Could not load test file '%s'. Error='%s'. Exiting.\n",
            filename,
            parameterXMLDoc.ErrorDesc() );
		return;
	}

  TiXmlNode* dataNode = parameterXMLDoc.FirstChild( "data" );
  if (  dataNode == 0  ) {
    printf("data node not found\n");
    return;
  }

  LoadObjects(dataNode->FirstChild("Objects"));
}

////////////////////////////////////////////////////////////////////////////////
void PhyskoEngine::LoadObjects(TiXmlNode* objectsNode)
{
  if ( objectsNode == 0 )
  {
    printf("object node not found\n");
    return;
  }

  // load spring
  TiXmlNode* objNode;
  objNode = objectsNode->FirstChild("Spring");
  while ( objNode != 0 )
  {
    m_objectsList.push_back((IObject*) new Spring(objNode->ToElement()));
    objNode = objNode->NextSibling("Spring");
  }

  // load cloth
  objNode = objectsNode->FirstChild("Cloth");
  while ( objNode != 0 )
  {
    m_objectsList.push_back((IObject*) new Cloth());
    TiXmlElement* clothElement = objNode->ToElement();
    int iVal;
    if ( clothElement->Attribute("value", &iVal) != NULL )
      printf("Load Cloth Number %i\n",iVal);

    objNode = objNode->NextSibling("Cloth");
  }

  // load animated cloth
  objNode = objectsNode->FirstChild("AnimatedCloth");
  while ( objNode != 0 )
  {
    m_objectsList.push_back((IObject*) new AnimatedCloth());
    TiXmlElement* animatedClothElement = objNode->ToElement();
    int iVal;
    if ( animatedClothElement->Attribute("value", &iVal) != NULL )
      printf("Load AnimatedCloth Number %i\n",iVal);

    objNode = objNode->NextSibling("AnimatedCloth");
  }
}







